import assert from "assert";
import {
  createUser,
  createUnit,
  getUsers,
  getImprovementWorks,
  getObjectById,
  createHospital,
  createSuggestion,
  getSuggestions,
  createImprovementWork,
  createTaskOnImprovementWork,
  updateInstance,
  deleteInstance,
} from "../client/Database/database.js";

describe("User and Unit Functions", function () {
  // Test to create a new unit
  it("should create a new unit in the database", async function () {
    try {
      const ID = "unit_012jkl";
      const hospitalId = "elasmo";
      const name = "Dermatology Clinic";

      await createUnit(ID, hospitalId, name);

      // Verify that the unit is created by filtering units based on hospitalId
      const unitsInHospital = await getObjectById("unit", ID);
      assert.ok(unitsInHospital, "Unit not found in the hospital");
      assert.strictEqual(unitsInHospital.name, name, "Unexpected unit name");
    } catch (error) {
      assert.fail(`Unexpected error: ${error}`);
    }
  });

  // Test to create a new user
  it("should create a new user in the database", async function () {
    try {
      const HSAId = "eribe255";
      const unitId = "unit_012jkl";
      const firstName = "erik";
      const lastName = "bertilsson";
      const email = "erik@example.com";
      const password = "password123";
      const role = "projektledare";
      const manager = true;

      let uid = "9wLmhunlsSWVFPQZ1cA1CcXEJMu1";
      await createUser(
        HSAId,
        unitId,
        firstName,
        lastName,
        email,
        manager,
        role,
        uid
      );

      // Verify that the user is created by filtering users based on unitId
      const usersInUnit = await getUsers({ unitId: unitId });
      const createdUser = usersInUnit.find((user) => user.id === HSAId);
      assert.ok(createdUser, "User not found in the unit");
      assert.strictEqual(
        createdUser.firstName,
        firstName,
        "Unexpected first name"
      );
      assert.strictEqual(
        createdUser.lastName,
        lastName,
        "Unexpected last name"
      );
      assert.strictEqual(createdUser.email, email, "Unexpected email");
    } catch (error) {
      assert.fail(`Unexpected error: ${error}`);
    }
  });

  // Test to create a new hospital
  it("should create a new hospital in the database", async function () {
    try {
      const ID = "elasmo";
      const name = "Elasmo General Hospital";

      await createHospital(ID, name);

      // Verify that the hospital is created by filtering hospitals
      const createdHospital = await getObjectById("hospital", ID); // Get all improvementWorks for simplicity
      assert.ok(createdHospital, "Hospital not found");
      assert.strictEqual(
        createdHospital.name,
        name,
        "Unexpected hospital name"
      );
    } catch (error) {
      assert.fail(`Unexpected error: ${error}`);
    }
  });

  // Test to create a new suggestion
  it("should create a new suggestion in the database", async function () {
    try {
      const id = "suggestion_001";
      const creatorHsaId = "eribe255";
      const name = "Improvement Idea";
      const content = "Implement a new process for better efficiency";
      const purpose = "purpose";
      const measure = "measure";
      const ideas = "idea";
      const unitId = "unit_abc123";
      const area = "area";
      const departmentId = "nvbgsh";
      const isPublic = true;

      await createSuggestion(
        id,
        creatorHsaId,
        name,
        area,
        content,
        purpose,
        measure,
        ideas,
        unitId,
        departmentId,
        isPublic
      );

      // Verify that the suggestion is created by filtering suggestions
      const suggestions = await getSuggestions({
        creatorHsaId: creatorHsaId,
        unitId: unitId,
        isPublic: isPublic,
      });
      const createdSuggestion = suggestions.find(
        (suggestion) => suggestion.id === id
      );
      assert.ok(createdSuggestion, "Suggestion not found");
      assert.strictEqual(
        createdSuggestion.name,
        name,
        "Unexpected suggestion name"
      );
      assert.strictEqual(
        createdSuggestion.content,
        content,
        "Unexpected suggestion content"
      );
      assert.strictEqual(
        createdSuggestion.unitId,
        unitId,
        "Unexpected unitId value"
      );
      assert.strictEqual(
        createdSuggestion.public,
        isPublic,
        "Unexpected isPublic value"
      );
    } catch (error) {
      assert.fail(`Unexpected error: ${error}`);
    }
  });

  // Test to create a new improvementWork
  it("should create a new improvementWork in the database", async function () {
    try {
      const id = "proj_0002";
      const name = "New Project";
      const currentStage = "Planning";
      const content = "Details about the new improvementWork";
      const unitId = "unit_abc123";
      const purpose = "purpose";

      await createImprovementWork(
        id,
        name,
        currentStage,
        content,
        unitId,
        purpose,
        "",
        "",
        "",
        [""],
        "",
        "",
        "",
        "",
        "",
        "",
        "",
        "",
        "",
        "",
        "",
        "",
        "",
        "",
        "",
        "",
        "",
        ""
      );
      // Verify that the improvementWork is created by filtering improvementWorks
      const improvementWorks = await getImprovementWorks({
        nolanStage: currentStage,
        unitId: unitId,
        completed: false,
      });
      const createdProject = improvementWorks.find(
        (improvementWork) => improvementWork.id === id
      );
      assert.ok(createdProject, "Project not found");
      assert.strictEqual(
        createdProject.name,
        name,
        "Unexpected improvementWork name"
      );
      assert.strictEqual(
        createdProject.currentStage,
        currentStage,
        "Unexpected currentStage value"
      );
      assert.strictEqual(
        createdProject.content,
        content,
        "Unexpected improvementWork content"
      );
      assert.strictEqual(
        createdProject.unitId,
        unitId,
        "Unexpected unitId value"
      );
      assert.strictEqual(
        createdProject.completed,
        false,
        "Unexpected completed value"
      );
    } catch (error) {
      assert.fail(`Unexpected error: ${error}`);
    }
  });
  it("should create a new task on a improvementWork in the database", async function () {
    try {
      const task_id = "task_006";
      const improvementWorkId = "proj_002";
      const name = "New Task";
      const deadlineDate = new Date("2024-10-16");
      const startDate = new Date("2022-10-16");
      const stage = "Planning";
      const content = "Details about the new task";

      await createTaskOnImprovementWork(
        task_id,
        improvementWorkId,
        name,
        deadlineDate,
        startDate,
        stage,
        content
      );

      // Verify that the task is created by getting task
      const createdTask = await getObjectById("task", task_id);
      assert.ok(createdTask, "Task not found");
      assert.strictEqual(createdTask.name, name, "Unexpected task name");
      assert.equal(
        createdTask.deadlineDate.seconds * 1000,
        deadlineDate.getTime(),
        "Unexpected deadlineDate value" +
          createdTask.deadlineDate.seconds * 1000 +
          "vs " +
          deadlineDate.getTime()
      );
      assert.equal(
        createdTask.startDate.seconds * 1000,
        startDate.getTime(),
        "Unexpected startDate value" +
          createdTask.startDate.seconds * 1000 +
          "vs " +
          startDate.getTime()
      );
      assert.strictEqual(createdTask.stage, stage, "Unexpected stage value");
      assert.strictEqual(
        createdTask.content,
        content,
        "Unexpected task content"
      );
    } catch (error) {
      assert.fail(`Unexpected error: ${error}`);
    }
  });

  // Test to update a user's information
  it("should update a user's information in the database", async function () {
    try {
      const HSAId = "eribe255";
      const updates = {
        firstName: "bert",
        lastName: "eriksson",
        email: "bert@example.net",
      };

      await updateInstance("users", HSAId, updates);

      // Verify that the user is updated by getting the updated user
      const updatedUser = await getObjectById("users", HSAId);
      assert.ok(updatedUser, "User not found");
      assert.strictEqual(
        updatedUser.firstName,
        updates.firstName,
        "Unexpected updated firstName"
      );
      assert.strictEqual(
        updatedUser.lastName,
        updates.lastName,
        "Unexpected updated lastName"
      );
      assert.strictEqual(
        updatedUser.email,
        updates.email,
        "Unexpected updated email"
      );
    } catch (error) {
      assert.fail(`Unexpected error: ${error}`);
    }
  });
});
describe("Database Cleanup", function () {
  // Replace the following arrays with the IDs of the instances created during the tests
  const userHSAIdsToDelete = "eribe255";
  const unitIdsToDelete = "unit_012jkl";
  const improvementWorkIdsToDelete = "proj_0002";
  const taskIdsToDelete = "task_006";
  const suggestionIdsToDelete = "suggestion_001";

  // Test to delete users
  it("should delete added users from the database", async function () {
    try {
      await deleteInstance("users", userHSAIdsToDelete);

      // Verify that users are deleted
      const deletedUser = await getObjectById("users", userHSAIdsToDelete);
      assert.strictEqual(
        deletedUser,
        null,
        `Deleted user ${userHSAIdsToDelete} still found`
      );
    } catch (error) {
      assert.fail(`Unexpected error: ${error}`);
    }
  });

  // Test to delete units
  it("should delete added units from the database", async function () {
    try {
      await deleteInstance("unit", unitIdsToDelete);

      // Verify that units are deleted
      const deletedUnit = await getObjectById("unit", unitIdsToDelete);
      assert.strictEqual(
        deletedUnit,
        null,
        `Deleted unit ${unitIdsToDelete} still found`
      );
    } catch (error) {
      assert.fail(`Unexpected error: ${error}`);
    }
  });

  // Test to delete improvementWorks
  it("should delete added improvementWorks from the database", async function () {
    try {
      await deleteInstance("improvementWork", improvementWorkIdsToDelete);

      // Verify that improvementWorks are deleted
      const deletedImprovementWork = await getObjectById(
        "improvementWork",
        improvementWorkIdsToDelete
      );
      assert.strictEqual(
        deletedImprovementWork,
        null,
        `Deleted improvementWork ${improvementWorkIdsToDelete} still found`
      );
    } catch (error) {
      assert.fail(`Unexpected error: ${error}`);
    }
  });

  // Test to delete tasks
  it("should delete added tasks from the database", async function () {
    try {
      await deleteInstance("task", taskIdsToDelete);

      // Verify that tasks are deleted
      const deletedTask = await getObjectById("task", taskIdsToDelete);
      assert.strictEqual(
        deletedTask,
        null,
        `Deleted task ${taskIdsToDelete} still found`
      );
    } catch (error) {
      assert.fail(`Unexpected error: ${error}`);
    }
  });

  // Test to delete suggestions
  it("should delete added suggestions from the database", async function () {
    try {
      await deleteInstance("suggestion", suggestionIdsToDelete);

      // Verify that suggestions are deleted
      const deletedSuggestion = await getObjectById(
        "suggestion",
        suggestionIdsToDelete
      );
      assert.strictEqual(
        deletedSuggestion,
        null,
        `Deleted suggestion ${suggestionIdsToDelete} still found`
      );
    } catch (error) {
      assert.fail(`Unexpected error: ${error}`);
    }
  });
});
